package de.meinefirma.meinprojekt.entities;

import javax.persistence.*;
import java.io.Serializable;
import java.sql.Timestamp;
import java.util.*;

@Entity
@Table( uniqueConstraints=@UniqueConstraint(columnNames={"teamName"}) )
@NamedQuery( name=Team.QUERY_BY_NAME, query="Select t from Team t where t.teamName = :tname" )
public class Team implements Serializable
{
   private static final long   serialVersionUID  = 1L;
   public  static final String QUERY_BY_NAME     = "Team.queryByName";
   public  static final String HTML_TABLE_HEADER = "<tr><th>Id</th><th>LastUpdate</th><th>Name</th><th>Person.Name</th></tr>\n";

   @Id @GeneratedValue( strategy = GenerationType.AUTO )
   private Long id;

   @Version
   private Timestamp lastUpdate;

   @Column( nullable = false, length = 200 )
   private String teamName;

   @OneToMany( mappedBy="team", cascade=CascadeType.PERSIST, fetch=FetchType.EAGER )
   private Set<Person> personen = new HashSet<>();

   public Set<Person> getPersonen() {
      return Collections.unmodifiableSet( personen );
   }

   public boolean addPerson( Person person ) {
      person.setTeam( this );
      return personen.add( person );
   }

   public boolean removePerson( Person person ) {
      boolean b = personen.remove( person );
      if( b ) person.setTeam( null );
      return b;
   }

   // Getter/Setter:
   public Long      getId()         { return id;         }
   public Timestamp getLastUpdate() { return lastUpdate; }
   public String    getName()       { return teamName;   }
   public void setId(         Long      id         ) { this.id = id;                 }
   public void setLastUpdate( Timestamp lastUpdate ) { this.lastUpdate = lastUpdate; }
   public void setName(       String    name       ) { this.teamName = name;         }

   @Override public boolean equals( Object obj ) {
      if( !(obj instanceof Team) ) return false;
      String onm = ((Team) obj).getName();
      return teamName == onm || (teamName != null && teamName.equals( onm ));
   }

   @Override public int hashCode() {
      return ( teamName == null ) ? 0 : teamName.hashCode();
   }

   @Override public String toString() {
      String ps = personenToString( personen );
      return superToString( super.toString() ) +
             ": id=" + id + ", lastUpdate='" + lastUpdate + "', name='" + teamName + "', personen='" + ps + "'";
   }

   public String toHtmlTableRow() {
      String ps = personenToString( personen );
      return "<tr><td>" + id + "</td><td>" + lastUpdate + "</td><td>" + teamName + "</td><td>" + ps + "</td></tr>\n";
   }

   private static String personenToString( Set<Person> personen ) {
      StringBuilder sb = new StringBuilder();
      if( personen != null && !personen.isEmpty() )
         for( Person p : personen ) sb.append( p.getName() ).append( ", " );
      String ps = sb.toString();
      if( ps.length() > 2 ) ps = ps.substring( 0, ps.length() - 2 );
      if( ps.length() == 0 ) ps = "(keine Personen zugeordnet)";
      return ps;
   }

   private static String superToString( String s ) {
      return s.substring( s.lastIndexOf( '.' ) + 1, s.length() );
   }
}
