package de.meinefirma.meinprojekt.dao;

import java.util.Properties;
import javax.naming.*;
import javax.persistence.*;
import org.apache.openejb.api.LocalClient;

@LocalClient
public class JpaTestUtil
{
   @PersistenceContext private EntityManager em;

   /** Initialisierung von Datenbank, EntityManager und UserTransaction */
   public void setupDbEmUtx()
   {
      // Falls eine automatische Tabellenanlage nicht ueber die persistence.xml initiiert werden soll,
      // kann dies fuer Tests auch hier ueber Environmentvariablen vorgegeben werden, z.B. fuer EclipseLink so:
      System.getProperties().setProperty( "eclipselink.ddl-generation", "create-tables" );

      // InitialContext fuer OpenEJB-Container:
      Properties p = new Properties();
      p.put( Context.INITIAL_CONTEXT_FACTORY, "org.apache.openejb.client.LocalInitialContextFactory" );
      p.put( "jdbc/MeinDatasourceJndiName",   "new://Resource?type=DataSource" );

      // HSQLDB (in Memory):
      p.put( "jdbc/MeinDatasourceJndiName.JdbcDriver", "org.hsqldb.jdbcDriver" );
      p.put( "jdbc/MeinDatasourceJndiName.JdbcUrl",    "jdbc:hsqldb:mem:MeineDb" );
//    Derby (im target-Verzeichnis):
//    p.put( "jdbc/MeinDatasourceJndiName.JdbcDriver", "org.apache.derby.jdbc.EmbeddedDriver" );
//    p.put( "jdbc/MeinDatasourceJndiName.JdbcUrl",    "jdbc:derby:./target/Derby-DB/mydb;create=true" );
//    MySQL:
//    p.put( "jdbc/MeinDatasourceJndiName.JdbcDriver", "com.mysql.jdbc.Driver" );
//    p.put( "jdbc/MeinDatasourceJndiName.JdbcUrl",    "jdbc:mysql://localhost:3306/MeineDb" );
//    p.put( "jdbc/MeinDatasourceJndiName.UserName",   "root" );
//    p.put( "jdbc/MeinDatasourceJndiName.PassWord",   "" );
//    Oracle-DB:
//    p.put( "jdbc/MeinDatasourceJndiName.JdbcDriver", "oracle.jdbc.OracleDriver" );
//    p.put( "jdbc/MeinDatasourceJndiName.JdbcUrl",    "jdbc:oracle:thin:@localhost:1521:XE" );
//    p.put( "jdbc/MeinDatasourceJndiName.Username",   "xx..." );
//    p.put( "jdbc/MeinDatasourceJndiName.Password",   "yy..." );

      try {
         (new InitialContext( p )).bind( "inject", this );
         try {
            (new InitialContext()).bind( "java:comp/env/persistence/em", em );
         } catch( NameAlreadyBoundException e ) { /* ok */ }
      } catch( NamingException ex ) {
         throw new RuntimeException( ex );
      }
   }
}
